<?php

namespace App\Service;

use App\Model\Depart;
use App\Model\Employee;
use App\Model\EmployeeDepartPermission;
use App\Model\EmployeeManagerDepart;
use App\Model\EmployeeMenuPermission;
use App\Model\EmployeeRole;
use App\Model\EmployeeTeamPermission;
use App\Model\Role;
use App\Model\RoleMenu;
use App\Model\RoleMenuButton;
use App\Model\SysMenu;
use App\Model\SysMenuButton;
use App\Model\Team;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Mockery\Exception;

/**
 * 人员相关
 * @package App\Models
 */
class EmployeeService extends Service
{
    public function employeeEdit($data,$user){
        list($status,$msg) = $this->employeeRule($data,false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();
            $model = new Employee();
            $model = $model->where('id',$data['id'])->first();
            $model->number = $data['number'];
            $model->emp_name = $data['emp_name'];
            $model->mobile = $data['mobile'] ?? '';
            $model->leave_time = $data['leave_time'] ?? '';
            $model->entry_time = $data['entry_time'] ?? '';
            $model->is_admin = $data['is_admin'];
            if($model->is_admin == 1){
                $model->account = $data['number'];
                if($data['password'] !== '******'){
                    $model->password   = Hash::make($data['password']);
                }
            }
            $model->save();

            EmployeeDepartPermission::where('employee_id',$data['id'])->delete();
            if(isset($data['depart'])){
                $insert = [];
                foreach ($data['depart'] as $value){
                    $insert[] = [
                        'employee_id' => $model->id,
                        'depart_id' => $value,
                    ];
                }
                EmployeeDepartPermission::insert($insert);
            }

            EmployeeRole::where('employee_id',$data['id'])->update([
                'del_time' => time()
            ]);
            if(isset($data['role'])){
                $insert = [];
                foreach ($data['role'] as $value){
                    $insert[] = [
                        'employee_id' => $model->id,
                        'role_id' => $value,
                        'crt_time' => time(),
                        'upd_time' => time(),
                    ];
                }
                EmployeeRole::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false, $exception->getMessage()];
        }

        return [true,''];
    }

    public function employeeAdd($data,$user){
        list($status,$msg) = $this->employeeRule($data);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $model = new Employee();

            $model->number = $data['number'];
            $model->emp_name = $data['emp_name'];
            $model->mobile = $data['mobile'] ?? '';
            $model->leave_time = $data['leave_time'] ?? '';
            $model->entry_time = $data['entry_time'] ?? '';
            $model->state = 1;
            $model->crt_id = $user['id'];
            $model->is_admin = $data['is_admin'];
            if($model->is_admin == 1){
                $model->account = $data['number'];
                if($data['password'] !== '********'){
                    $model->password   = Hash::make($data['password']);
                }
            }
            $model->save();

            if(isset($data['depart'])){
                $insert = [];
                foreach ($data['depart'] as $value){
                    $insert[] = [
                        'employee_id' => $model->id,
                        'depart_id' => $value,
                    ];
                }
                EmployeeDepartPermission::insert($insert);
            }

            if(isset($data['role'])){
                $insert = [];
                foreach ($data['role'] as $value){
                    $insert[] = [
                        'employee_id' => $model->id,
                        'role_id' => $value,
                        'crt_time' => time(),
                        'upd_time' => time(),
                    ];
                }
                EmployeeRole::insert($insert);
            }

            DB::commit();
        }catch (Exception $e){
            DB::rollBack();
            return [false, $e->getMessage()];
        }

        return [true,''];
    }

    public function employeeDel($data){
        if($this->isEmpty($data,'id')) return [false,'请选择删除的数据！'];

        Employee::whereIn('id',$data['id'])->update([
            'del_time'=>time()
        ]);

        return [true,'删除成功'];
    }

    public function employeeList($data,$user){
        $model = Employee::where('del_time',0)
            ->select('number','mobile','emp_name','id','entry_time','leave_time','is_admin','state')
            ->orderBy('id','desc');

        if(! empty($data['depart'])) {
            $employee_id = DB::table('employee_depart_permission')
                ->where("depart_id", $data['depart'])
                ->select("employee_id")
                ->get()->toArray();
            $employee_id = array_column($employee_id,'employee_id');
            $model->whereIn("id", $employee_id);
        }
        if(! empty($data['number'])) $model->where('number', 'LIKE', '%'.$data['number'].'%');
        if(! empty($data['emp_name'])) $model->where('emp_name', 'LIKE', '%'.$data['emp_name'].'%');
        if(! empty($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['mobile'])) $model->where('mobile', 'LIKE', '%'.$data['mobile'].'%');
        if(! isset($data['all_emp'])) $model->where('id','<>',Employee::SPECIAL_ADMIN);
        if(! empty($data['role'])) {
            $emp = EmployeeRole::where('role_id',$data['role'])
                ->where('del_time',0)
                ->select('employee_id')->get()->toArray();
            $model->whereIn('id',array_column($emp,'employee_id'));
        }
        $list = $this->limit($model,'',$data);

        //组织数据
        $list = $this->organizationEmployeeData($list);

        return [true , $list];
    }

    public function organizationEmployeeData($data) {
        if (empty($data['data'])) return $data;

        $res = DB::table('employee_role as a')
            ->leftJoin('role as b','a.role_id','=','b.id')
            ->where('a.del_time',0)
            ->where('b.del_time',0)
            ->whereIn("a.employee_id",array_column($data['data'],'id'))
            ->select('a.employee_id','b.title','b.id')
            ->get()->toArray();
        $role = $role2 = [];
        foreach ($res as $value){
            if(isset($role[$value->employee_id])){
                $role[$value->employee_id] .= ',' . $value->title;
            }else{
                $role[$value->employee_id] = $value->title;
            }
            $role2[$value->employee_id][] = $value->id;
        }

        $res = DB::table('employee_depart_permission as a')
            ->select('a.employee_id','b.title','b.id')
            ->join('depart as b','a.depart_id','=','b.id')
            ->whereIn("a.employee_id",array_column($data['data'],'id'))
            ->orderBy('b.id')
            ->get()->toArray();
        $depart_title = $depart_id = [];
        foreach ($res as $value){
            if(isset($depart_title[$value->employee_id])){
                $depart_title[$value->employee_id] .= ',' . $value->title;
            }else{
                $depart_title[$value->employee_id] = $value->title;
            }
            $depart_id[$value->employee_id][] = $value->id;
        }

        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['role'] = $role2[$value['id']] ?? [];
            $data['data'][$key]['role_name'] = $role[$value['id']] ?? '';
            $data['data'][$key]['depart'] = $depart_id[$value['id']] ?? [];
            $data['data'][$key]['depart_title'] = $depart_title[$value['id']] ?? '';
        }
        return $data;
    }

    public function employeeRule($data,$is_add = true){
        if($this->isEmpty($data,'number')) return [false,'工号不存在！'];
        if($this->isEmpty($data,'emp_name')) return [false,'姓名不存在！'];

        if(! $is_add){
            if($this->isEmpty($data,'id')) return [false,'ID不能为空！'];
            $bool = Employee::where('number',$data['number'])
                ->where('id','<>',$data['id'])
                ->where('del_time',0)->exists();
        }else{
            $bool = Employee::where('number',$data['number'])
                ->where('del_time',0)->exists();
        }
        if($bool) return [false,'工号已存在！'];

        return [true,''];
    }

    public function roleEdit($data){
        list($status,$msg) = $this->roleRule($data, false);
        if(!$status) return [$status,$msg];

        $model = new Role();
        $model = $model->where('id',$data['id'])->first();
        $model->title = $data['title'];
        $model->save();

        return [true,'保存成功！'];
    }

    public function roleAdd($data,$user){
        list($status,$msg) = $this->roleRule($data);
        if(!$status) return [$status,$msg];

        $model = new Role();
        $model->title = $data['title'] ;
        $model->save();

        return [true,'保存成功！'];
    }

    public function roleDel($data){
        if($this->isEmpty($data,'id')) return [false,'ID必须！'];

        $bool = EmployeeRole::where('del_time',0)
            ->whereIn('role_id',$data['id'])
            ->exists();
        if($bool) return [false,'角色已绑定人员！'];

        Role::where('id',$data['id'])->update([
            'del_time' => time()
        ]);

        return [true,'删除成功'];
    }

    public function roleList($data){
        $list = Role::where('del_time',0)
            ->select('title','crt_time','id','upd_time')
            ->orderBy('id','desc');
        if(! empty($data['title'])) $list->where('title', 'LIKE', '%' . $data['title'] . '%');

        $list = $this->limit($list,'',$data);

        return [200,$list];
    }

    public function roleRule($data,$is_check = true){
        if($this->isEmpty($data,'title')) return [false,'名称不能为空！'];

        if($is_check){
            $bool = Role::where('title',$data['title'])
                ->where('del_time',0)
                ->exists();
            if($bool) return [false,'角色名称已存在！'];
        }else{
            if($this->isEmpty($data,'id')) return [false,'ID不能为空！'];
            $bool = Role::where('title',$data['title'])
                ->where('id','<>',$data['id'])
                ->where('del_time',0)
                ->exists();
            if($bool) return [false,'角色名称已存在！'];
        }

        return [true,''];
    }

    public function roleMenu($data){
        if(empty($data['role_id']))  return [false,'角色不能为空！'];
        if(empty($data['menu'])) return [false,'菜单数据不能为空！'];

        DB::beginTransaction();
        try {
            RoleMenu::where('role_id',$data['role_id'])->update(['del_time' => time()]);
            RoleMenuButton::where('role_id',$data['role_id'])->update(['del_time' => time()]);

            $insert = $insert2 = [];
            foreach ($data['menu'] as $t){
                $insert[] = [
                    'role_id' => $data['role_id'],
                    'menu_id' => $t['menu_id'],
                    'type' => $t['type'],
                    'crt_time' => time()
                ];
                if(! empty($t['button'])){
                    foreach ($t['button'] as $b){
                        $insert2[] = [
                            'role_id' => $data['role_id'],
                            'menu_id' => $t['menu_id'],
                            'button_id' => $b,
                            'crt_time' => time()
                        ];
                    }
                RoleMenuButton::insert($insert2);
                }
            }
            RoleMenu::insert($insert);

            DB::commit();
        }catch (\Throwable $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,'保存成功!'];
    }

    public function roleDetail($data){
        if(empty($data['role_id'])) return [false,'请选择角色'];

        $role = Role::where('id',$data['role_id'])
            ->where('del_time',0)
            ->select('id','code','title')
            ->first();
        if(empty($role)) return [false,'角色不存在或已被删除'];
        $role = $role->toArray();

        $menu = RoleMenu::where('role_id',$data['role_id'])
            ->where('del_time',0)
            ->select('menu_id','type')
            ->get()->toArray();
        $button = $this->fillRoleButton([$data['role_id']]);
        foreach ($menu as $key => $value){
            $menu[$key]['button'] = $button[$value['menu_id']] ?? [];
        }
        $role['menu'] = $menu;

        return [true, $role];
    }

    public function departEdit($data){
        list($status,$msg) = $this->departRule($data,false);
        if(!$status) return [$status,$msg];

        $update = $msg['data'][0];

        $model = new Depart();
        $model->where('id',$data['id'])->update($update);

        return [true,'保存成功！'];
    }

    public function departAdd($data,$user){
        list($status,$msg) = $this->departRule($data);
        if(!$status) return [$status,$msg];

        Depart::insert($msg['data']);

        return [true,'保存成功！'];
    }

    public function departDel($data){
        list($status,$msg) = $this->checkDepartDel($data);
        if(! $status) return [false, $msg];

        Depart::whereIn('id',$data['id'])->update([
            'del_time'=>time()
        ]);

        return [true,'删除成功'];
    }

    public function checkDepartDel($data){
        if($this->isEmpty($data,'id')) return [false,'ID必须！'];

        $bool = Depart::whereIn('parent_id',$data['id'])->where('del_time',0)->exists();
        if($bool) return [false,'部门下有子部门！'];

        if($this->checkDepartHasPerson($data['id'])) return [false,'部门下有人员档案！'];

        return [true, ''];
    }

    public function departList($data,$user){
        $model = Depart::where('del_time',0)
            ->select('title','id','code','parent_id','is_main')
            ->orderby('code', 'asc');
        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');
        if(! empty($data['code'])) $model->where('code', 'LIKE', '%'.$data['code'].'%');

        $list = $model->get()->toArray();
        $list_tree = $list;
        if(! empty($list_tree)) {
            $list_tree = $this->makeTree(0,$list_tree);
            $list_tree = $this->set_sort_circle($list_tree);
        }

        return [200,['data' => $list,'tree' => $list_tree]];
    }

    public function departRule($data, $is_check = true){
        if($this->isEmpty($data,'data')) return [false,'数据不能为空！'];

        $code = array_column($data['data'],'code');
        $title = array_column($data['data'],'title');
        $code = array_map(function($val) {
            return $val !== null ? $val : 0;
        }, $code);
        $title = array_map(function($val) {
            return $val !== null ? $val : 0;
        }, $title);
        $code_count = array_count_values($code);
        $title_count = array_count_values($title);
        foreach ($code as $value){
            if(empty($value)) return [false,'编码不能为空！'];
            if($code_count[$value] > 1) return [false,'编码不能重复'];
        }
        foreach ($title as $value){
            if(empty($value)) return [false,'名称不能为空！'];
            if($title_count[$value] > 1) return [false,'名称不能重复'];
        }

        foreach ($data['data'] as $key => $value){
            if(empty($value['parent_id'])) $data['data'][$key]['parent_id'] = 0;

            $data['data'][$key]['upd_time'] = time();

            if($is_check){
                $data['data'][$key]['crt_time'] = time();
                $bool = Depart::whereRaw("(binary code = '{$value['code']}' OR title = '{$value['title']}')")
                    ->where('del_time',0)
                    ->exists();
            }else{
                if($this->isEmpty($data,'id')) return [false,'id不能为空！'];
                $bool = Depart::whereRaw("(binary code = '{$value['code']}' OR title = '{$value['title']}')")
                    ->where('id','<>',$data['id'])
                    ->where('del_time',0)
                    ->exists();
            }
            if($bool) return [false,'编码或部门名称不能重复'];
        }

        return [true, $data];
    }

    //检测部门下是否存在人员
    public function checkDepartHasPerson($depart_id = []){
        if(empty($depart_id)) return false;

        $bool = EmployeeDepartPermission::from('employee_depart_permission as a')
            ->leftJoin('employee as b','b.id','a.employee_id')
            ->where('b.del_time',0)
            ->whereIn('a.depart_id',$depart_id)
            ->exists();

        return $bool;
    }

    public function teamEdit($data){
        list($status,$msg) = $this->teamRule($data,false);
        if(!$status) return [$status,$msg];

        $model = new Team();
        $model = $model->where('id',$data['id'])->first();
        $model->title = $data['title'];
        $model->code = $data['code'];
        $model->save();

        return [true,'保存成功！'];
    }

    public function teamAdd($data,$user){
        list($status,$msg) = $this->teamRule($data);
        if(!$status) return [$status,$msg];

        $model = new Team();
        $model->title = $data['title'] ;
        $model->code = $data['code'];
        $model->save();

        return [true,'保存成功！'];
    }

    public function teamDel($data){
        if($this->isEmpty($data,'id')) return [false,'ID必须！'];

        Team::where('id',$data['id'])->update([
            'del_time'=>time()
        ]);

        return [true,'删除成功'];
    }

    public function teamList($data){
        $list = Team::where('del_time',0)
            ->select('title','id','crt_time','upd_time','code')
            ->orderBy('id','desc');
        $list = $this->limit($list,'',$data);

        return [200,$list];
    }

    public function teamRule($data,$is_add = true){
        if($this->isEmpty($data,'title')) return [false,'名称不存在！'];
        if($this->isEmpty($data,'code')) return [false,'编码不存在'];

        $model = Team::where('title',$data['title'])
            ->where('code',$data['code'])
            ->where('del_time',0);
        if(! $is_add){
            if($this->isEmpty($data,'id')) return [false,'ID不能为空'];
            $model->where('id','<>',$data['id']);
        }
        $bool = $model->exists();
        if($bool) return [false,'名称和编码已存在！'];

        return [true,''];
    }

    public function teamDetail($data){
        if($this->isEmpty($data,'id')) return [false,'ID不能为空！'];

        $result = EmployeeTeamPermission::from('employee_team_permission as a')
            ->leftJoin('employee as b','b.id','a.employee_id')
            ->where('team_id',$data['id'])
            ->select('b.id','b.emp_name','b.number as code')
            ->get()->toArray();

        return [true,$result];
    }

    public function employeeRole($data){
        $role_ids = [];
        $employee_ids = [];
        foreach ($data as $v){
            if(isset($v['role_id'])){
                if(!in_array($v['role_id'],$role_ids)){
                    $role_ids[] = $v['role_id'];
                }
            }

            if(isset($v['employee_id'])){
                if(!in_array($v['employee_id'],$employee_ids)){
                    $employee_ids[] = $v['employee_id'];
                }
            }
        }

        EmployeeMenuPermission::wherein('role_id',$role_ids)->delete();
        EmployeeMenuPermission::wherein('employee_id',$employee_ids)->delete();


        EmployeeMenuPermission::insert($data);

        return [200,'保存成功!'];

    }

    public function employeeDepart($data){
        if($this->isEmpty($data,'insert')) return [false,'数据不能为空！'];

        DB::beginTransaction();
        try {
            if($data['type'] == 1){
                EmployeeDepartPermission::whereIn('depart_id',$data['insert']['depart_id'])->delete();
            }else{
                EmployeeDepartPermission::whereIn('employee_id',$data['insert']['employee_id'])->delete();
            }

            $insert = [];
            foreach ($data['insert']['depart_id'] as $t){
                foreach ($data['insert']['employee_id'] as $e){
                    $insert[] = [
                        'depart_id' => $t,
                        'employee_id' => $e
                    ];
                }
            }
            EmployeeDepartPermission::insert($insert);

            DB::commit();
        }catch (\Throwable $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,'保存成功!'];
    }

    public function employeeTeam($data){
        if($this->isEmpty($data,'insert')) return [false,'数据不能为空！'];

        DB::beginTransaction();
        try {
            if($data['type'] == 1){
                EmployeeTeamPermission::whereIn('team_id',$data['insert']['team_id'])->delete();
            }else{
                EmployeeTeamPermission::whereIn('employee_id',$data['insert']['employee_id'])->delete();
            }

            $insert = [];
            foreach ($data['insert']['team_id'] as $t){
                foreach ($data['insert']['employee_id'] as $e){
                    $insert[] = [
                        'team_id' => $t,
                        'employee_id' => $e
                    ];
                }
            }
            EmployeeTeamPermission::insert($insert);

            DB::commit();
        }catch (\Throwable $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,'保存成功!'];
    }

    public function loginRule($data){
        if($this->isEmpty($data,'account')) return [false,'账号不能为空！'];
        if($this->isEmpty($data,'password')) return [false,'密码不存在！'];

        $res = Employee::where('del_time',0)
            ->where('account', $data['account'])
            ->get()->toArray();

        if(empty($res)) return [false,'账号不存在或已被删除！'];

        $res = reset($res);
        if(! Hash::check($data['password'], $res['password'])) return [false,'密码错误！'];
        if($res['is_admin'] != Employee::IS_ADMIN) return [false,'该账号不能登录！'];
        if($res['state'] == Employee::NOT_USE) return [false,'账号停用！'];

        return [true, ['id'=>$res['id'], 'name'=>$res['emp_name'], 'account' => $res['account']]];
    }

    public static function checkUser($userId){
        $res = Employee::where('id', $userId)
            ->where('del_time',0)
            ->where('is_admin',Employee::IS_ADMIN)
            ->where('state',Employee::USE)->get()->first();
        if(empty($res)) return [false, '该账号无法登录,请联系管理员！'];

        return [true, $res];
    }

    //获取登录账号的角色
    public static function getPersonRole($employee_id){
        if(empty($employee_id)) return [];
        $role = EmployeeRole::where('del_time',0)
            ->where('employee_id',$employee_id)
            ->select('role_id')
            ->get()->toArray();

        //组织
        $role_id = array_unique(array_column($role,'role_id'));
        asort($role_id);
        $role_id = array_values($role_id);

        return $role_id;
    }

    //获取登录账号的角色的菜单
    public function getMenuByRole($role_id,$user_id){
        $menu = SysMenu::where('del_time',0)->select('id')->get()->toArray();
        $button = SysMenuButton::where('del_time',0)->select('id','menu_id')->get()->toArray();
        $button_map = [];
        foreach ($button as $value){
            $button_map[$value['menu_id']][] = $value['id'];
        }
        $object = [];//返回的模型
        if($user_id == Employee::SPECIAL_ADMIN){
            //超级管理员
            foreach ($menu as $value){
                $object[] = [
                    'type' => 0,//所有权限
                    'menu_id' => $value['id'],
                    'button' => $button_map[$value['id']] ?? [],
                ];
            }
            return $object;
        }

        //没绑定角色
        if(empty($role_id)) return [];

        $search = RoleMenu::whereIn('role_id',$role_id)
            ->where('del_time',0)
            ->select('menu_id','type')
            ->get()->toArray();
        $button = $this->fillRoleButton($role_id);
        $tmp = [];
        foreach ($search as $value){
            if(! in_array($value['menu_id'],$tmp)){
                $object[] = [
                    'menu_id' => $value['menu_id'],
                    'type' => $value['type'],
                    'button' => $button[$value['menu_id']] ?? [],
                ];
                $tmp[] = $value['menu_id'];
            }
        }
        unset($tmp);

        return $object;
    }

    //人员直接绑定部门
    public function employeeManagerDepart($data,$user){
        if($user['id'] != Employee::SPECIAL_ADMIN) return [false,'非ADMIN账号不能操作'];

        if($this->isEmpty($data,'employee_id')) return [false,'请选择操作人员'];
        if($this->isEmpty($data,'depart_id')) return [false,'请选择部门'];

        EmployeeManagerDepart::where('employee_id',$data['employee_id'])->update([
            'del_time' => time()
        ]);

        $insert = [];
        foreach ($data['depart_id'] as $value){
            $insert[] = [
                'employee_id' => $data['employee_id'],
                'depart_id' => $value,
                'crt_time' => time(),
                'upd_time' => time(),
            ];
        }
        EmployeeManagerDepart::insert($insert);

        return [true,''];
    }

    //填充角色下的按钮
    public function fillRoleButton($role_id){
        $button = RoleMenuButton::whereIn('role_id',$role_id)
            ->where('del_time',0)
            ->select('menu_id','button_id')
            ->get()->toArray();
        $button_map = [];
        foreach ($button as $value){
            if(! isset($button_map[$value['menu_id']])){
                $button_map[$value['menu_id']][] = $value['button_id'];
            }else{
                if(! in_array($value['button_id'], $button_map[$value['menu_id']])) $button_map[$value['menu_id']][] = $value['button_id'];
            }
        }

        return $button_map;
    }

    //获取登录账号的部门
    public static function getLoginDepart($employee_id){
        if(empty($employee_id)) return ['', '','',''];

        //admin账号
        if($employee_id == Employee::SPECIAL_ADMIN) return [Depart::RULE_DEPART, '','',''];

        //自己绑定的部门 启用的部门
        $depart = EmployeeDepartPermission::from('employee_depart_permission as a')
            ->join('depart as b','b.id','a.depart_id')
            ->where('a.employee_id',$employee_id)
            ->where('b.is_use',Depart::IS_UES)
            ->select('a.depart_id','b.is_main','b.parent_id')
            ->orderBy('a.depart_id','asc')
            ->get()->toArray();

        $top = $map = $rule = [];
        if(! empty($depart)){
            $list = Depart::where('del_time',0)->get()->toArray();
            $depart_map = array_column($list,null,'id');
            foreach ($depart as $value){
                if($value['parent_id'] == 0){//顶级
                    $tmp['depart_id'] = $value['depart_id'];
                    $tmp['is_main'] = $value['is_main'];
                    $top[] = $tmp;
                    $map[$value['depart_id']] = $value['depart_id'];
                }else{
                    $t = self::getTopParentId($value['depart_id'],$list);
                    if($t && isset($depart_map[$t])) {
                        $tmp['depart_id'] = $depart_map[$t]['id'];
                        $tmp['is_main'] = $depart_map[$t]['is_main'];
                        $top[] = $tmp;
                        $map[$value['depart_id']] = $t;
                    }
                }
            }
            foreach ($depart as $value){
                if(in_array($value['depart_id'],$rule)) continue;
                if(! $value['parent_id']){ //顶级
                    if($value['is_main']) {//是总公司
                        //所有部门都有
                        $rule = array_column($list,'id');
                    }else{//不是总公司
                        //自己以及子部门
                        $depart_id = array_merge(self::getAllIds($list,$map[$value['depart_id']]),[$map[$value['depart_id']]]);
                        $rule = array_merge_recursive($rule,$depart_id);
                    }
                }else{//非顶级
                    if($value['is_main']) {//是总社
                        $top_tmp = $map[$value['depart_id']];
                        if(! empty($depart_map[$top_tmp]['is_main'])){
                            //顶级部门是总公司  所有部门都有
                            $rule = array_column($list,'id');
                        }else{
                            //顶级部门不是总公司 自己以及自己之下
                            $depart_id = array_merge(self::getAllIds($list,$value['depart_id']),[$value['depart_id']]);
                            $rule = array_merge_recursive($rule,$depart_id);
                        }
                    }else{//不是总社
                        $rule = array_merge($rule,[$value['depart_id']]);
                    }
                }
            }
        }

        return [$depart, $top, $map, array_unique($rule)];
    }

    public static function getTopParentId($id, $data) {
        foreach ($data as $item) {
            if ($item['id'] == $id) {
                if ($item['parent_id'] == 0) {
                    // 找到最顶级的id
                    return $item['id'];
                } else {
                    // 继续递归查找父级
                    return self::getTopParentId($item['parent_id'], $data);
                }
            }
        }

        // 如果没有找到匹配的id，则返回null或者其他你希望的默认值
        return 0;
    }

    public static function getAllIds($data, $id) {
        $result = array(); // 存储结果的数组

        foreach ($data as $node) {
            if ($node['parent_id'] == $id) { // 如果当前节点的父 ID 等于指定 ID，则将该节点添加到结果中
                $result[] = $node['id'];
                // 递归查询该节点的所有子孙节点，并将结果合并到结果数组中
                $result = array_merge($result, self::getAllIds($data, $node['id']));
            }
        }

        return $result;
    }
}
