<?php

namespace App\Service;

use App\Model\Depart;
use App\Model\Employee;
use App\Model\PurchaseOrder;
use App\Model\PurchaseOrderInfo;
use Illuminate\Support\Facades\DB;

class PurchaseOrderService extends Service
{
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,false);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = PurchaseOrder::where('order_number',$data['order_number'])->first();
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->contact_order_no = $data['contact_order_no'] ?? '';
            $material_model->mark = $data['mark'] ?? '';
            $material_model->save();

            PurchaseOrderInfo::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'order_number' => $data['order_number'],
                        'depart_id' => $data['depart_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data);
        if(!$status) return [$status,$msg];
        try{
            DB::beginTransaction();
            $material_model = new PurchaseOrder();
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->contact_order_no = $data['contact_order_no'] ?? '';
            $material_model->mark = $data['mark'] ?? '';
            $material_model->crt_id = $user['id'];
            $material_model->save();

            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'order_number' => $data['order_number'],
                        'depart_id' => $data['depart_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function detail($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->select('id','order_number','depart_id','mark','state','crt_id','order_type','contact_order_no')
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];

        $info = PurchaseOrderInfo::where('del_time',0)
            ->where('order_number',$data['order_number'])
            ->select('id','order_number','depart_id','product_id','number','mark')
            ->get()->toArray();
        $order['product'] = $info;

        return [true, $order];
    }

    public function del($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据！'];

        $bool = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('state','>',PurchaseOrder::STATE_ZERO)->exists();
        if($bool) return [false,'采购订单已审核,删除失败！'];

        try{
            DB::beginTransaction();
            PurchaseOrder::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);
            PurchaseOrderInfo::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    public function getList($data,$user){
        $model = PurchaseOrder::where('del_time',0)
            ->select('order_number','id','depart_id','mark','crt_time','state','crt_id','order_type','contact_order_no')
            ->orderby('id', 'desc');
        if(! empty($data['order_number'])) $model->where('order_number', 'LIKE', '%'.$data['order_number'].'%');
        if(! empty($data['depart_id'])) $model->where('depart_id', $data['depart_id']);
        if(! empty($data['company_id'])) $model->where('company_id', $data['company_id']);
        if(isset($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->whereBetween('crt_time',[$return[0],$return[1]]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list);

        return [true, $list];
    }

    public function fillListData($data){
        if(empty($data['data'])) return $data;

        $depart_id = array_unique(array_column($data['data'],'depart_id'));
        $depart_map = Depart::whereIn('id',$depart_id)
            ->pluck('title','id')
            ->toArray();

        $emp_id = array_unique(array_column($data['data'],'crt_id'));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();

        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['depart_name'] = $depart_map[$value['depart_id']] ?? '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['order_type_title'] = PurchaseOrder::$order_type[$value['order_type']] ?? '';
        }

        return $data;
    }

    public function orderRule(&$data, $is_check = true){
        if(empty($data['order_type'])) return [false,'采购订单类型不能为空'];
        if($this->isEmpty($data,'product')) return [false,'产品不能为空'];

        foreach ($data['product'] as $value){
            if(empty($value['number'])) return [false,'产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的产品数量'];
        }

        if($is_check){
            $order_number = (new OrderNoService())->createOrderNumber(PurchaseOrder::prefix);
            if(empty($order_number)) return [false,'采购单号生成失败！'];
            $data['order_number'] = $order_number;
        }else{
            if($this->isEmpty($data,'order_number')) return [false,'采购单号不能为空！'];
        }

        return [true, ''];
    }

    public function purchaseOrderConfirm($data,$user){
        if(empty($data['order_number'])) return [false, '请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        if($order->state != PurchaseOrder::STATE_ZERO) return [false, '请确认采购订单状态，操作失败'];

        PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->update(['state' => PurchaseOrder::STATE_ONE]);

        return [true,''];
    }
}
