<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\Customer;
use App\Model\Employee;
use App\Model\Product;
use App\Model\SalesOrder;
use App\Model\SalesOrderInfo;
use App\Model\SalesOrderProductInfo;
use Illuminate\Support\Facades\DB;

class SalesOrderService extends Service
{
    public function salesOrderEdit($data,$user){
        list($status,$msg) = $this->salesOrderRule($data, $user, false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = SalesOrder::where('id', $data['id'])->first();
            $model->sales_order_type = $data['sales_order_type'];
            $model->model_type = $data['model_type'];
            $model->order_number = $data['order_number'];
            $model->title = $data['title'] ?? '';
            $model->selling_price = $data['selling_price'] ?? 0;
            $model->vin_no = $data['vin_no'] ?? '';
            $model->order_type = $data['order_type'] ?? 0;
            $model->deal_type = $data['deal_type'] ?? 0;
            $model->customer_id = $data['customer_id'] ?? 0;
            $model->sign_time = $data['sign_time'] ?? 0;
            $model->contract_state = $data['contract_state'] ?? 0;
            $model->product_total = $data['product_total'] ?? 0;
            $model->rate = $data['rate'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->construction_time = $data['construction_time'] ?? 0;
            $model->handover_time = $data['handover_time'] ?? 0;
            $model->expire_time = $data['expire_time'] ?? 0;
            $model->other_fee = $data['other_fee'] ?? 0;
            $model->discount_fee = $data['discount_fee'] ?? 0;
            $model->contract_fee = $data['contract_fee'] ?? 0;
            $model->contract_type = $data['contract_type'] ?? 0;
            $model->pay_way = $data['pay_way'] ?? 0;
            $model->send_state = $data['send_state'] ?? 0;
            $model->logistics_company = $data['logistics_company'] ?? 0;
            $model->logistics_number = $data['logistics_number'] ?? '';
            $model->car_type = $data['car_type'] ?? 0;
            $model->year = $data['year'] ?? '';
            $model->mileage = $data['mileage'] ?? '';
            $model->color = $data['color'] ?? '';
            $model->original_set = $data['original_set'] ?? '';
            $model->processing = $data['processing'] ?? '';
            $model->depart_id = $data['depart_id'] ?? 0;
            $model->top_depart_id = $data['top_depart_id'] ?? 0;
            $model->save();
            $time = time();

            SalesOrderInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => $time]);
            SalesOrderProductInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => $time]);

            if(! empty($data['employee_one'])){
                $insert = [];
                foreach ($data['employee_one'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'employee_id' => $value,
                        'type' => SalesOrderInfo::type_one,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_two'])){
                $insert = [];
                foreach ($data['employee_two'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'employee_id' => $value,
                        'type' => SalesOrderInfo::type_two,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_three'])){
                $insert = [];
                foreach ($data['employee_three'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'employee_id' => $value,
                        'type' => SalesOrderInfo::type_three,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['product'])){
                $insert = [];
                foreach ($data['product'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'product_id' => $value['product_id'],
                        'price' => $value['retail_price'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'crt_time' => $time,
                    ];
                }
                SalesOrderProductInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function salesOrderAdd($data,$user){
        list($status,$msg) = $this->salesOrderRule($data,$user);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = new SalesOrder();
            $model->sales_order_type = $data['sales_order_type'];
            $model->model_type = $data['model_type'];
            $model->order_number = $data['order_number'];
            $model->title = $data['title'] ?? '';
            $model->selling_price = $data['selling_price'] ?? 0;
            $model->vin_no = $data['vin_no'] ?? '';
            $model->order_type = $data['order_type'] ?? 0;
            $model->deal_type = $data['deal_type'] ?? 0;
            $model->customer_id = $data['customer_id'] ?? 0;
            $model->sign_time = $data['sign_time'] ?? 0;
            $model->contract_state = $data['contract_state'] ?? 0;
            $model->product_total = $data['product_total'] ?? 0;
            $model->rate = $data['rate'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->construction_time = $data['construction_time'] ?? 0;
            $model->handover_time = $data['handover_time'] ?? 0;
            $model->expire_time = $data['expire_time'] ?? 0;
            $model->other_fee = $data['other_fee'] ?? 0;
            $model->discount_fee = $data['discount_fee'] ?? 0;
            $model->contract_fee = $data['contract_fee'] ?? 0;
            $model->contract_type = $data['contract_type'] ?? 0;
            $model->pay_way = $data['pay_way'] ?? 0;
            $model->send_state = $data['send_state'] ?? 0;
            $model->logistics_company = $data['logistics_company'] ?? 0;
            $model->logistics_number = $data['logistics_number'] ?? '';
            $model->car_type = $data['car_type'] ?? 0;
            $model->year = $data['year'] ?? '';
            $model->mileage = $data['mileage'] ?? '';
            $model->color = $data['color'] ?? '';
            $model->original_set = $data['original_set'] ?? '';
            $model->processing = $data['processing'] ?? '';
            $model->depart_id = $data['depart_id'] ?? 0;
            $model->top_depart_id = $data['top_depart_id'] ?? 0;
            $model->crt_id = $user['id'];
            $model->save();
            $time = time();

            if(! empty($data['employee_one'])){
                $insert = [];
                foreach ($data['employee_one'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'employee_id' => $value,
                        'type' => SalesOrderInfo::type_one,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_two'])){
                $insert = [];
                foreach ($data['employee_two'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'employee_id' => $value,
                        'type' => SalesOrderInfo::type_two,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['employee_three'])){
                $insert = [];
                foreach ($data['employee_three'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'employee_id' => $value,
                        'type' => SalesOrderInfo::type_three,
                        'crt_time' => $time,
                    ];
                }
                SalesOrderInfo::insert($insert);
            }

            if(! empty($data['product'])){
                $insert = [];
                foreach ($data['product'] as $value){
                    $insert[] = [
                        'sales_order_id' => $model->id,
                        'product_id' => $value['product_id'],
                        'price' => $value['retail_price'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                        'crt_time' => $time,
                    ];
                }
                SalesOrderProductInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getLine().':'.$exception->getMessage()];
        }

        return [true,''];
    }

    public function salesOrderDel($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        try {
            DB::beginTransaction();

            SalesOrder::where('id',$data['id'])->update([
                'del_time'=> time()
            ]);
            SalesOrderInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => time()]);
            SalesOrderProductInfo::where('del_time',0)
                ->where('sales_order_id',$data['id'])
                ->update(['del_time' => time()]);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function salesOrderDetail($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        $sales = SalesOrder::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($sales)) return [false,'销售订单不存在或已被删除'];
        $sales = $sales->toArray();
        $customer = Customer::where('id',$sales['customer_id'])->value('title');
        $sales['customer_title'] = $customer;
        $sales['employee_one'] = $sales['employee_two'] = $sales['employee_three'] = $sales['product'] = [];
        $array = [
            $sales['order_type'],
            $sales['deal_type'],
            $sales['contract_state'],
            $sales['contract_type'],
            $sales['car_type'],
            $sales['pay_way'],
            $sales['send_state'],
            $sales['logistics_company'],
        ];
        $basic_map = BasicType::whereIn('id',$array)
            ->pluck('title','id')
            ->toArray();
        $sales = [$sales];
        foreach ($sales as $key => $value){
            $sales[$key]['order_type_title'] = $basic_map[$value['order_type']] ?? '';
            $sales[$key]['deal_type_title'] = $basic_map[$value['deal_type']] ?? '';
            $sales[$key]['contract_state_title'] = $basic_map[$value['contract_state']] ?? '';
            $sales[$key]['contract_type_title'] = $basic_map[$value['contract_type']] ?? '';
            $sales[$key]['car_type_title'] = $basic_map[$value['car_type']] ?? '';
            $sales[$key]['pay_way_title'] = $basic_map[$value['pay_way']] ?? '';
            $sales[$key]['send_state_title'] = $basic_map[$value['send_state']] ?? '';
            $sales[$key]['logistics_company_title'] = $basic_map[$value['logistics_company']] ?? '';
        }
        $sales = $sales[0];

        $sales_info = SalesOrderInfo::where('del_time',0)
            ->where('sales_order_id',$sales['id'])
            ->select('id','sales_order_id','employee_id','type')
            ->get()->toArray();
        $emp_map = Employee::whereIn('id',array_unique(array_merge_recursive([$sales['crt_id']],array_column($sales_info,'employee_id'))))
            ->pluck('emp_name','id')
            ->toArray();
        foreach ($sales_info as $value){
            $tmp = [
                'id' => $value['employee_id'],
                'name' => $emp_map[$value['employee_id']],
            ];
            if($value['type'] == SalesOrderInfo::type_one){
                $sales['employee_one'][] = $tmp;
            }elseif ($value['type'] == SalesOrderInfo::type_two){
                $sales['employee_two'][] = $tmp;
            }elseif ($value['type'] == SalesOrderInfo::type_three){
                $sales['employee_three'][] = $tmp;
            }
        }

        $sales_p_info = SalesOrderProductInfo::where('del_time',0)
            ->where('sales_order_id',$sales['id'])
            ->select('id','sales_order_id','product_id','mark','price','number')
            ->get()->toArray();
        $pro = Product::whereIn('id',array_column($sales_p_info,'product_id'))
            ->select('bar_code','code','cost','depart_price','size','title','id')
            ->get()->toArray();
        $pro = array_column($pro,null,'id');
        foreach ($sales_p_info as $value){
            $p = $pro[$value['product_id']] ?? [];
            $sales['product'][] = [
                'product_id' => $value['product_id'],
                'mark' => $value['mark'],
                'retail_price' => $value['price'],
                'number' => $value['number'],
                'bar_code' => $p['bar_code'] ?? '',
                'code' => $p['code'] ?? '',
                'cost' => $p['cost'] ?? 0,
                'depart_price' => $p['depart_price'] ?? 0,
                'size' => $p['size'] ?? '',
                'title' => $p['title'] ?? '',
            ];
        }
        $sales['crt_name'] = $emp_map[$sales['crt_id']] ?? '';
        $sales['crt_time'] = $sales['crt_time'] ? date("Y-m-d H:i:s",$sales['crt_time']): '';

        return [true, $sales];
    }

    public function salesOrderList($data,$user){
        $model = new SalesOrder(['userData' => $user]);
        $model = $model->where('del_time',0)
            ->select('title','id','sales_order_type','model_type','order_number','selling_price','vin_no','car_type','order_type','deal_type','customer_id','sign_time','contract_state','crt_id','crt_time','mark','product_total','rate','construction_time','handover_time','expire_time','other_fee','discount_fee','contract_fee','contract_type','pay_way','send_state','logistics_company','logistics_number','car_type','year','mileage','color','original_set','processing','state')
            ->orderby('id', 'desc');

        if(! empty($data['sales_order_type'])) $model->where('sales_order_type',$data['sales_order_type']);
        if(! empty($data['title'])) $model->where('title', 'LIKE', '%'.$data['title'].'%');
        if(! empty($data['model_type'])) $model->where('model_type',$data['model_type']);
        if(! empty($data['time_type'])) {
            if($data['time_type'] == 1) {
                $start = strtotime('today');
                $end = strtotime('tomorrow') - 1;
            }elseif ($data['time_type'] == 2){
                $start = strtotime('this week',strtotime('today'));
                $end = strtotime('this week +6 days 23:59:59', strtotime('today'));
            }
            if(! empty($start) && ! empty($end)) {
                $model->where('crt_time','>=',$start);
                $model->where('crt_time','<=',$end);
            }
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillData($list,$data);

        return [true, $list];
    }

    public function salesOrderRule(&$data, $user, $is_add = true){
        if(empty($data['sales_order_type'])) return [false,'合同类型不能为空'];
        if(empty($data['model_type'])) return [false,'订单模板类型不能为空'];
        if(! in_array($data['model_type'],SalesOrder::$model_type)) return [false,'订单模板类型错误'];
        if(empty($data['order_number'])) return [false,'合同编号不能为空'];
        if(empty($data['customer_id'])) return [false,'请选择客户'];
        if(empty($data['employee_one'])) return [false,'请选择签订人'];
        if(empty($data['sign_time'])) return [false,'签订日期不能为空'];
        $data['sign_time'] = $this->changeDateToDateMin($data['sign_time']);
        if(! empty($data['expire_time'])) $data['expire_time'] = $this->changeDateToDateMin($data['expire_time']);
        if(empty($data['contract_state'])) return [false,'请选择合同状态'];
        if(empty($data['employee_two'])) return [false,'请选择负责人'];
        if(empty($data['product'])) return [false,'请选择产品'];
        foreach ($data['product'] as $value){
            if(empty($value['number'])) return [false,'产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的产品数量'];

            if(empty($value['retail_price'])) return [false,'产品零售价不能为空'];
            $res = $this->checkNumber($value['retail_price']);
            if(! $res) return [false,'产品零售价请输入不超过两位小数并且大于0的数值'];
        }
        if(empty($data['contract_fee'])) return [false,'合同金额不能为空'];
        $res = $this->checkNumber($data['contract_fee']);
        if(! $res) return [false, '合同金额请输入不超过两位小数并且大于0的数值'];
        if(! empty($data['rate'])){
            $res = $this->checkNumber($data['rate']);
            if(! $res) return [false, '整单扣除率请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['other_fee'])){
            $res = $this->checkNumber($data['other_fee']);
            if(! $res) return [false, '其它费用请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['discount_fee'])){
            $res = $this->checkNumber($data['discount_fee']);
            if(! $res) return [false, '优惠金额请输入不超过两位小数并且大于0的数值'];
        }
        //所属部门 以及  顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        if($data['model_type'] == SalesOrder::Model_type_one){
            if(empty($data['order_type'])) return [false,'订单类型不能为空'];
            if(empty($data['construction_time'])) return [false,'施工日期不能为空'];
            $data['construction_time'] = $this->changeDateToDateMin($data['construction_time']);
            if(empty($data['handover_time'])) return [false,'交车日期不能为空'];
            $data['handover_time'] = $this->changeDateToDateMin($data['handover_time']);
            if(empty($data['deal_type'])) return [false,'成交类型不能为空'];
        }elseif($data['model_type'] == SalesOrder::Model_type_two){
            if(empty($data['title'])) return [false,'合同名称不能为空'];
        }else{
            if(empty($data['car_type'])) return [false,'车型不能为空'];
            if(empty($data['year'])) return [false,'年份不能为空'];
            if(empty($data['mileage'])) return [false,'表显里程不能为空'];
        }

        if($is_add){
            $bool = SalesOrder::where('del_time',0)->where('order_number',$data['order_number'])->exists();
            if($bool) return [false,'合同单号已存在，请重新获取'];

            if($data['model_type'] == SalesOrder::Model_type_two){
                $boolean = SalesOrder::where('del_time',0)
                    ->where('title',$data['title'])
                    ->where('model_type',$data['model_type'])
                    ->exists();
            }
        }else{
            if(empty($data['id'])) return [false,'ID不能为空'];
            if($data['model_type'] == SalesOrder::Model_type_two){
                $boolean = SalesOrder::where('del_time',0)
                    ->where('id','<>',$data['id'])
                    ->where('title',$data['title'])
                    ->where('model_type',$data['model_type'])
                    ->exists();
            }
        }
        if(isset($boolean) && $boolean) return [false,'合同名称不能重复'];

        return [true, $data];
    }

    public function fillData($data,$ergs){
        if(empty($data['data'])) return $data;

        $array = array_unique(array_merge_recursive(array_column($data['data'],'order_type'),array_column($data['data'],'deal_type'),array_column($data['data'],'contract_state'),array_column($data['data'],'car_type'),array_column($data['data'],'pay_way'),array_column($data['data'],'logistics_company'),array_column($data['data'],'send_state'),array_column($data['data'],'contract_type')));
        $basic_map = BasicType::whereIn('id',$array)
            ->pluck('title','id')
            ->toArray();
        $emp = Employee::whereIn('id',array_unique(array_column($data['data'],'crt_id')))
            ->pluck('emp_name','id')
            ->toArray();
        $customer = Customer::whereIn('id',array_unique(array_column($data['data'],'customer_id')))
            ->pluck('title','id')
            ->toArray();

        $product = [];
        if(! empty($ergs['product'])){
            $sales_p_info = SalesOrderProductInfo::where('del_time',0)
                ->where('sales_order_id',array_column($data['data'],'id'))
                ->select('id','sales_order_id','product_id','mark','price','number')
                ->get()->toArray();
            $pro = Product::whereIn('id',array_column($sales_p_info,'product_id'))
                ->select('bar_code','code','cost','depart_price','size','title','id')
                ->get()->toArray();
            $pro = array_column($pro,null,'id');
            foreach ($sales_p_info as $value){
                $p = $pro[$value['product_id']] ?? [];
                $tmp = [
                    'product_id' => $value['product_id'],
                    'mark' => $value['mark'],
                    'retail_price' => $value['price'],
                    'number' => $value['number'],
                    'bar_code' => $p['bar_code'] ?? '',
                    'code' => $p['code'] ?? '',
                    'cost' => $p['cost'] ?? 0,
                    'depart_price' => $p['depart_price'] ?? 0,
                    'size' => $p['size'] ?? '',
                    'title' => $p['title'] ?? '',
                ];
                $product[$value['sales_order_id']][] = $tmp;
            }
        }
        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['sales_order_type_title'] = SalesOrder::$order_type[$value['sales_order_type']] ?? '';
            $data['data'][$key]['order_type_title'] = $basic_map[$value['order_type']] ?? '';
            $data['data'][$key]['deal_type_title'] = $basic_map[$value['deal_type']] ?? '';
            $data['data'][$key]['contract_state_title'] = $basic_map[$value['contract_state']] ?? '';
            $data['data'][$key]['contract_type_title'] = $basic_map[$value['contract_type']] ?? '';
            $data['data'][$key]['car_type_title'] = $basic_map[$value['car_type']] ?? '';
            $data['data'][$key]['pay_way_title'] = $basic_map[$value['pay_way']] ?? '';
            $data['data'][$key]['send_state_title'] = $basic_map[$value['send_state']] ?? '';
            $data['data'][$key]['logistics_company_title'] = $basic_map[$value['logistics_company']] ?? '';
            $data['data'][$key]['customer_title'] = $customer[$value['customer_id']] ?? '';
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date('Y-m-d H:i:s',$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp[$value['crt_id']] ?? '';
            $data['data'][$key]['product'] = $product[$value['id']] ?? [];
        }

        return $data;
    }

    public function salesOrderGet($data){
        if(empty($data['model_type'])) return [false,'订单模板类型不能为空'];
        if(! isset(SalesOrder::$prefix[$data['model_type']])) return [false,'订单模板类型错误'];
        $prefix = SalesOrder::$prefix[$data['model_type']];
        $order_number = OrderNoService::createSalesOrderNumber($prefix);
        if(! $order_number) return [false,'合同编号生成失败！'];

        return [true, ['order_number' => $order_number]];
    }

    public function salesOrderDispatch($data){
//        list($status,$msg)
    }

    public function salesOrderDispatchRule($data){

        return [true,''];
    }
}
